import { ɵɵdefineInjectable, ɵɵinject, NgZone, Injectable, Optional, EventEmitter as EventEmitter$1, Component, Input, Output, ViewChild, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { EventEmitter } from 'events';
import { CommonModule } from '@angular/common';

function loadScript(src) {
    return new Promise((resolve, reject) => {
        const script = createScriptElement();
        script.src = src;
        script.onload = () => {
            resolve(script);
        };
        script.onerror = () => {
            reject(new Error(`Failed to load ${src}`));
        };
        getHeadElement().appendChild(script);
    });
}
function createScriptElement() {
    const s = document.createElement('script');
    s.async = true;
    return s;
}
function getHeadElement() {
    return document.head || document.getElementsByTagName('head')[0];
}

/*
yt-player. MIT License. Feross Aboukhadijeh <https://feross.org/opensource>

The MIT License (MIT)

Copyright (c) Feross Aboukhadijeh

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
const YOUTUBE_IFRAME_API_SRC = 'https://www.youtube.com/iframe_api';
const YOUTUBE_STATES = {
    '-1': 'unstarted',
    0: 'ended',
    1: 'playing',
    2: 'paused',
    3: 'buffering',
    5: 'cued',
};
const YOUTUBE_ERROR = {
    // The request contains an invalid parameter value. For example, this error
    // occurs if you specify a videoId that does not have 11 characters, or if the
    // videoId contains invalid characters, such as exclamation points or asterisks.
    INVALID_PARAM: 2,
    // The requested content cannot be played in an HTML5 player or another error
    // related to the HTML5 player has occurred.
    HTML5_ERROR: 5,
    // The video requested was not found. This error occurs when a video has been
    // removed (for any reason) or has been marked as private.
    NOT_FOUND: 100,
    // The owner of the requested video does not allow it to be played in embedded
    // players.
    UNPLAYABLE_1: 101,
    // This error is the same as 101. It's just a 101 error in disguise!
    UNPLAYABLE_2: 150,
};
const loadIframeAPICallbacks = [];
class YouTubePlayer extends EventEmitter {
    constructor(element, opts) {
        super();
        const elem = typeof element === 'string' ? document.querySelector(element) : element;
        if (elem.id) {
            this._id = elem.id; // use existing element id
        }
        else {
            this._id = elem.id = 'ytplayer-' + Math.random().toString(16).slice(2, 8);
        }
        this._opts = Object.assign({
            width: 640,
            height: 360,
            autoplay: false,
            captions: undefined,
            controls: true,
            keyboard: true,
            fullscreen: true,
            annotations: true,
            modestBranding: false,
            related: true,
            timeupdateFrequency: 1000,
            playsInline: true,
            start: 0,
        }, opts);
        this.videoId = null;
        this.destroyed = false;
        this._api = null;
        this._autoplay = false; // autoplay the first video?
        this._player = null;
        this._ready = false; // is player ready?
        this._queue = [];
        this._interval = null;
        // Setup listeners for 'timeupdate' events. The YouTube Player does not fire
        // 'timeupdate' events, so they are simulated using a setInterval().
        this._startInterval = this._startInterval.bind(this);
        this._stopInterval = this._stopInterval.bind(this);
        super.on('playing', this._startInterval);
        super.on('unstarted', this._stopInterval);
        super.on('ended', this._stopInterval);
        super.on('paused', this._stopInterval);
        super.on('buffering', this._stopInterval);
        this._loadIframeAPI((err, api) => {
            if (err) {
                return this._destroy(new Error('YouTube Iframe API failed to load'));
            }
            this._api = api;
            // If load(videoId, [autoplay, [size]]) was called before Iframe API
            // loaded, ensure it gets called again now
            if (this.videoId) {
                this.load(this.videoId, this._autoplay, this._start);
            }
        });
    }
    get _window() {
        return window;
    }
    load(videoId, autoplay = false, start = 0) {
        if (this.destroyed) {
            return;
        }
        this.videoId = videoId;
        this._autoplay = autoplay;
        this._start = start;
        // If the Iframe API is not ready yet, do nothing. Once the Iframe API is
        // ready, `load(this.videoId)` will be called.
        if (!this._api) {
            return;
        }
        // If there is no player instance, create one.
        if (!this._player) {
            this._createPlayer(videoId);
            return;
        }
        // If the player instance is not ready yet, do nothing. Once the player
        // instance is ready, `load(this.videoId)` will be called. This ensures that
        // the last call to `load()` is the one that takes effect.
        if (!this._ready) {
            return;
        }
        // If the player instance is ready, load the given `videoId`.
        if (autoplay) {
            this._player.loadVideoById(videoId, start);
        }
        else {
            this._player.cueVideoById(videoId, start);
        }
    }
    play() {
        if (this._ready) {
            this._player.playVideo();
        }
        else {
            this._queueCommand('play');
        }
    }
    pause() {
        if (this._ready) {
            this._player.pauseVideo();
        }
        else {
            this._queueCommand('pause');
        }
    }
    stop() {
        if (this._ready) {
            this._player.stopVideo();
        }
        else {
            this._queueCommand('stop');
        }
    }
    seek(seconds) {
        if (this._ready) {
            this._player.seekTo(seconds, true);
        }
        else {
            this._queueCommand('seek', seconds);
        }
    }
    setVolume(volume) {
        if (this._ready) {
            this._player.setVolume(volume);
        }
        else {
            this._queueCommand('setVolume', volume);
        }
    }
    getVolume() {
        return (this._ready && this._player.getVolume()) || 0;
    }
    mute() {
        if (this._ready) {
            this._player.mute();
        }
        else {
            this._queueCommand('mute');
        }
    }
    unMute() {
        if (this._ready) {
            this._player.unMute();
        }
        else {
            this._queueCommand('unMute');
        }
    }
    isMuted() {
        return (this._ready && this._player.isMuted()) || false;
    }
    setSize(width, height) {
        if (this._ready) {
            this._player.setSize(width, height);
        }
        else {
            this._queueCommand('setSize', width, height);
        }
    }
    setPlaybackRate(rate) {
        if (this._ready) {
            this._player.setPlaybackRate(rate);
        }
        else {
            this._queueCommand('setPlaybackRate', rate);
        }
    }
    setPlaybackQuality(suggestedQuality) {
        if (this._ready) {
            this._player.setPlaybackQuality(suggestedQuality);
        }
        else {
            this._queueCommand('setPlaybackQuality', suggestedQuality);
        }
    }
    getPlaybackRate() {
        return (this._ready && this._player.getPlaybackRate()) || 1;
    }
    getAvailablePlaybackRates() {
        return (this._ready && this._player.getAvailablePlaybackRates()) || [1];
    }
    getDuration() {
        return (this._ready && this._player.getDuration()) || 0;
    }
    getProgress() {
        return (this._ready && this._player.getVideoLoadedFraction()) || 0;
    }
    getState() {
        return ((this._ready && YOUTUBE_STATES[this._player.getPlayerState()]) ||
            'unstarted');
    }
    getCurrentTime() {
        return (this._ready && this._player.getCurrentTime()) || 0;
    }
    destroy() {
        this._destroy();
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    _destroy(err) {
        if (this.destroyed) {
            return;
        }
        this.destroyed = true;
        if (this._player) {
            // tslint:disable-next-line: no-unused-expression
            this._player.stopVideo && this._player.stopVideo();
            this._player.destroy();
        }
        this.videoId = null;
        this._id = null;
        this._opts = null;
        this._api = null;
        this._player = null;
        this._ready = false;
        this._queue = null;
        this._stopInterval();
        super.removeListener('playing', this._startInterval);
        super.removeListener('paused', this._stopInterval);
        super.removeListener('buffering', this._stopInterval);
        super.removeListener('unstarted', this._stopInterval);
        super.removeListener('ended', this._stopInterval);
        if (err) {
            super.emit('error', err);
        }
    }
    _queueCommand(command, ...args) {
        if (this.destroyed) {
            return;
        }
        this._queue.push([command, args]);
    }
    _flushQueue() {
        while (this._queue.length) {
            const command = this._queue.shift();
            this[command[0]].apply(this, command[1]);
        }
    }
    _loadIframeAPI(cb) {
        // If API is loaded, there is nothing else to do
        if (this._window.YT && typeof this._window.YT.Player === 'function') {
            return cb(null, this._window.YT);
        }
        // Otherwise, queue callback until API is loaded
        loadIframeAPICallbacks.push(cb);
        const scripts = Array.from(document.getElementsByTagName('script'));
        const isLoading = scripts.some((script) => script.src === YOUTUBE_IFRAME_API_SRC);
        // If API <script> tag is not present in the page, inject it. Ensures that
        // if user includes a hardcoded <script> tag in HTML for performance, another
        // one will not be added
        if (!isLoading) {
            loadScript(YOUTUBE_IFRAME_API_SRC).catch((err) => {
                while (loadIframeAPICallbacks.length) {
                    const loadCb = loadIframeAPICallbacks.shift();
                    loadCb(err);
                }
            });
        }
        const prevOnYouTubeIframeAPIReady = this._window.onYouTubeIframeAPIReady;
        this._window.onYouTubeIframeAPIReady = () => {
            if (typeof prevOnYouTubeIframeAPIReady === 'function') {
                prevOnYouTubeIframeAPIReady();
            }
            while (loadIframeAPICallbacks.length) {
                const loadCb = loadIframeAPICallbacks.shift();
                loadCb(null, this._window.YT);
            }
        };
    }
    _createPlayer(videoId) {
        if (this.destroyed) {
            return;
        }
        const opts = this._opts;
        this._player = new this._api.Player(this._id, {
            width: opts.width,
            height: opts.height,
            videoId,
            // (Not part of documented API) This parameter controls the hostname that
            // videos are loaded from. Set to `'https://www.youtube-nocookie.com'`
            // for enhanced privacy.
            host: opts.host,
            playerVars: {
                // This parameter specifies whether the initial video will automatically
                // start to play when the player loads. Supported values are 0 or 1. The
                // default value is 0.
                autoplay: opts.autoplay ? 1 : 0,
                // Setting the parameter's value to 1 causes closed captions to be shown
                // by default, even if the user has turned captions off. The default
                // behavior is based on user preference.
                cc_load_policy: opts.captions != null ? (opts.captions !== false ? 1 : 0) : undefined,
                // Sets the player's interface language. The parameter value is an ISO
                // 639-1 two-letter language code or a fully specified locale. For
                // example, fr and fr-ca are both valid values. Other language input
                // codes, such as IETF language tags (BCP 47) might also be handled
                // properly.
                hl: opts.captions != null && opts.captions !== false
                    ? opts.captions
                    : undefined,
                // This parameter specifies the default language that the player will
                // use to display captions. Set the parameter's value to an ISO 639-1
                // two-letter language code.
                cc_lang_pref: opts.captions != null && opts.captions !== false
                    ? opts.captions
                    : undefined,
                // This parameter indicates whether the video player controls are
                // displayed. For IFrame embeds that load a Flash player, it also defines
                // when the controls display in the player as well as when the player
                // will load. Supported values are:
                //   - controls=0 – Player controls do not display in the player. For
                //                  IFrame embeds, the Flash player loads immediately.
                //   - controls=1 – (default) Player controls display in the player. For
                //                  IFrame embeds, the controls display immediately and
                //                  the Flash player also loads immediately.
                //   - controls=2 – Player controls display in the player. For IFrame
                //                  embeds, the controls display and the Flash player
                //                  loads after the user initiates the video playback.
                controls: opts.controls ? 2 : 0,
                // Setting the parameter's value to 1 causes the player to not respond to
                // keyboard controls. The default value is 0, which means that keyboard
                // controls are enabled.
                disablekb: opts.keyboard ? 0 : 1,
                // Setting the parameter's value to 1 enables the player to be
                // controlled via IFrame or JavaScript Player API calls. The default
                // value is 0, which means that the player cannot be controlled using
                // those APIs.
                enablejsapi: 1,
                // Setting this parameter to 0 prevents the fullscreen button from
                // displaying in the player. The default value is 1, which causes the
                // fullscreen button to display.
                fs: opts.fullscreen ? 1 : 0,
                // Setting the parameter's value to 1 causes video annotations to be
                // shown by default, whereas setting to 3 causes video annotations to not
                // be shown by default. The default value is 1.
                iv_load_policy: opts.annotations ? 1 : 3,
                // This parameter lets you use a YouTube player that does not show a
                // YouTube logo. Set the parameter value to 1 to prevent the YouTube logo
                // from displaying in the control bar. Note that a small YouTube text
                // label will still display in the upper-right corner of a paused video
                // when the user's mouse pointer hovers over the player.
                modestbranding: opts.modestBranding ? 1 : 0,
                // This parameter provides an extra security measure for the IFrame API
                // and is only supported for IFrame embeds. If you are using the IFrame
                // API, which means you are setting the enablejsapi parameter value to 1,
                // you should always specify your domain as the origin parameter value.
                origin: this._window.location.origin,
                // This parameter controls whether videos play inline or fullscreen in an
                // HTML5 player on iOS. Valid values are:
                //   - 0: This value causes fullscreen playback. This is currently the
                //        default value, though the default is subject to change.
                //   - 1: This value causes inline playback for UIWebViews created with
                //        the allowsInlineMediaPlayback property set to TRUE.
                playsinline: opts.playsInline ? 1 : 0,
                // This parameter indicates whether the player should show related
                // videos from the same channel (0) or from any channel (1) when
                // playback of the video ends. The default value is 1.
                rel: opts.related ? 1 : 0,
                // (Not part of documented API) Allow html elements with higher z-index
                // to be shown on top of the YouTube player.
                wmode: 'opaque',
                // This parameter causes the player to begin playing the video at the given number
                // of seconds from the start of the video. The parameter value is a positive integer.
                // Note that similar to the seek function, the player will look for the closest
                // keyframe to the time you specify. This means that sometimes the play head may seek
                // to just before the requested time, usually no more than around two seconds.
                start: opts.start,
            },
            events: {
                onReady: () => this._onReady(videoId),
                onStateChange: (data) => this._onStateChange(data),
                onPlaybackQualityChange: (data) => this._onPlaybackQualityChange(data),
                onPlaybackRateChange: (data) => this._onPlaybackRateChange(data),
                onError: (data) => this._onError(data),
            },
        });
    }
    /**
     * This event fires when the player has finished loading and is ready to begin
     * receiving API calls.
     */
    _onReady(videoId) {
        if (this.destroyed) {
            return;
        }
        this._ready = true;
        // Once the player is ready, always call `load(videoId, [autoplay, [size]])`
        // to handle these possible cases:
        //
        //   1. `load(videoId, true)` was called before the player was ready. Ensure that
        //      the selected video starts to play.
        //
        //   2. `load(videoId, false)` was called before the player was ready. Now the
        //      player is ready and there's nothing to do.
        //
        //   3. `load(videoId, [autoplay])` was called multiple times before the player
        //      was ready. Therefore, the player was initialized with the wrong videoId,
        //      so load the latest videoId and potentially autoplay it.
        this.load(this.videoId, this._autoplay, this._start);
        this._flushQueue();
    }
    /**
     * Called when the player's state changes. We emit friendly events so the user
     * doesn't need to use YouTube's YT.PlayerState.* event constants.
     */
    _onStateChange(data) {
        if (this.destroyed) {
            return;
        }
        const state = YOUTUBE_STATES[data.data];
        if (state) {
            // Send a 'timeupdate' anytime the state changes. When the video halts for any
            // reason ('paused', 'buffering', or 'ended') no further 'timeupdate' events
            // should fire until the video unhalts.
            if (['paused', 'buffering', 'ended'].includes(state)) {
                this._onTimeupdate();
            }
            super.emit(state);
            // When the video changes ('unstarted' or 'cued') or starts ('playing') then a
            // 'timeupdate' should follow afterwards (never before!) to reset the time.
            if (['unstarted', 'playing', 'cued'].includes(state)) {
                this._onTimeupdate();
            }
        }
        else {
            throw new Error('Unrecognized state change: ' + data);
        }
    }
    /**
     * This event fires whenever the video playback quality changes. Possible
     * values are: 'small', 'medium', 'large', 'hd720', 'hd1080', 'highres'.
     */
    _onPlaybackQualityChange(data) {
        if (this.destroyed) {
            return;
        }
        super.emit('playbackQualityChange', data.data);
    }
    /**
     * This event fires whenever the video playback rate changes.
     */
    _onPlaybackRateChange(data) {
        if (this.destroyed) {
            return;
        }
        super.emit('playbackRateChange', data.data);
    }
    /**
     * This event fires if an error occurs in the player.
     */
    _onError(data) {
        if (this.destroyed) {
            return;
        }
        const code = data.data;
        // The HTML5_ERROR error occurs when the YouTube player needs to switch from
        // HTML5 to Flash to show an ad. Ignore it.
        if (code === YOUTUBE_ERROR.HTML5_ERROR) {
            return;
        }
        // The remaining error types occur when the YouTube player cannot play the
        // given video. This is not a fatal error. Report it as unplayable so the user
        // has an opportunity to play another video.
        if (code === YOUTUBE_ERROR.UNPLAYABLE_1 ||
            code === YOUTUBE_ERROR.UNPLAYABLE_2 ||
            code === YOUTUBE_ERROR.NOT_FOUND ||
            code === YOUTUBE_ERROR.INVALID_PARAM) {
            return super.emit('unplayable', this.videoId);
        }
        // Unexpected error, does not match any known type
        this._destroy(new Error('YouTube Player Error. Unknown error code: ' + code));
    }
    /**
     * This event fires when the time indicated by the `getCurrentTime()` method
     * has been updated.
     */
    _onTimeupdate() {
        super.emit('timeupdate', this.getCurrentTime());
    }
    _startInterval() {
        this._interval = setInterval(() => this._onTimeupdate(), this._opts.timeupdateFrequency);
    }
    _stopInterval() {
        clearInterval(this._interval);
        this._interval = null;
    }
}

var YtPlayerEvent;
(function (YtPlayerEvent) {
    YtPlayerEvent["ERROR"] = "error";
    YtPlayerEvent["UNPLAYABLE"] = "unplayable";
    YtPlayerEvent["UNSTARTED"] = "unstarted";
    YtPlayerEvent["ENDED"] = "ended";
    YtPlayerEvent["PLAYING"] = "playing";
    YtPlayerEvent["PAUSED"] = "paused";
    YtPlayerEvent["BUFFERING"] = "buffering";
    YtPlayerEvent["CUED"] = "cued";
    YtPlayerEvent["PLAYBACK_QUALITY_CHANGE"] = "playbackQualityChange";
    YtPlayerEvent["PLAYBACK_RATE_CHANGE"] = "playbackRateChange";
    YtPlayerEvent["TIMEUPDATE"] = "timeupdate";
})(YtPlayerEvent || (YtPlayerEvent = {}));

var PlaybackQuality;
(function (PlaybackQuality) {
    PlaybackQuality["Small"] = "small";
    PlaybackQuality["Medium"] = "medium";
    PlaybackQuality["Large"] = "large";
    PlaybackQuality["Hd720"] = "hd720";
    PlaybackQuality["Hd1080"] = "hd1080";
    PlaybackQuality["HighRes"] = "highres";
    PlaybackQuality["Default"] = "default";
})(PlaybackQuality || (PlaybackQuality = {}));

var PlayerState;
(function (PlayerState) {
    PlayerState[PlayerState["Unstarted"] = 0] = "Unstarted";
    PlayerState[PlayerState["Ended"] = 1] = "Ended";
    PlayerState[PlayerState["Playing"] = 2] = "Playing";
    PlayerState[PlayerState["Paused"] = 3] = "Paused";
    PlayerState[PlayerState["Buffering"] = 4] = "Buffering";
    PlayerState[PlayerState["Cued"] = 5] = "Cued";
})(PlayerState || (PlayerState = {}));

var StateChangeType;
(function (StateChangeType) {
    StateChangeType[StateChangeType["Error"] = 0] = "Error";
    StateChangeType[StateChangeType["Unplayable"] = 1] = "Unplayable";
    StateChangeType[StateChangeType["Unstarted"] = 2] = "Unstarted";
    StateChangeType[StateChangeType["Ended"] = 3] = "Ended";
    StateChangeType[StateChangeType["Started"] = 4] = "Started";
    StateChangeType[StateChangeType["PlaybackProgress"] = 5] = "PlaybackProgress";
    StateChangeType[StateChangeType["Paused"] = 6] = "Paused";
    StateChangeType[StateChangeType["Buffering"] = 7] = "Buffering";
    StateChangeType[StateChangeType["Cued"] = 8] = "Cued";
    StateChangeType[StateChangeType["QualityChanged"] = 9] = "QualityChanged";
    StateChangeType[StateChangeType["RateChanged"] = 10] = "RateChanged";
})(StateChangeType || (StateChangeType = {}));

class EventsRegistry {
    constructor(ngZone) {
        this.ngZone = ngZone;
        this.stateChange$ = new Subject();
        this.eventHandlers = new Map([
            [
                YtPlayerEvent.ERROR,
                (err) => this.broadcastStatusChange(StateChangeType.Error, err),
            ],
            [
                YtPlayerEvent.UNPLAYABLE,
                (videoId) => this.broadcastStatusChange(StateChangeType.Unplayable, videoId),
            ],
            [
                YtPlayerEvent.UNSTARTED,
                () => this.broadcastStatusChange(StateChangeType.Unstarted),
            ],
            [
                YtPlayerEvent.ENDED,
                () => this.broadcastStatusChange(StateChangeType.Ended),
            ],
            [
                YtPlayerEvent.PLAYING,
                () => this.broadcastStatusChange(StateChangeType.Started),
            ],
            [
                YtPlayerEvent.PAUSED,
                () => this.broadcastStatusChange(StateChangeType.Paused),
            ],
            [
                YtPlayerEvent.BUFFERING,
                () => this.broadcastStatusChange(StateChangeType.Buffering),
            ],
            [
                YtPlayerEvent.CUED,
                () => this.broadcastStatusChange(StateChangeType.Cued),
            ],
            [
                YtPlayerEvent.PLAYBACK_QUALITY_CHANGE,
                (quality) => this.broadcastStatusChange(StateChangeType.QualityChanged, quality),
            ],
            [
                YtPlayerEvent.PLAYBACK_RATE_CHANGE,
                (rate) => this.broadcastStatusChange(StateChangeType.RateChanged, rate),
            ],
            [
                YtPlayerEvent.TIMEUPDATE,
                (time) => this.broadcastStatusChange(StateChangeType.PlaybackProgress, time),
            ],
        ]);
    }
    register(player) {
        this.eventHandlers.forEach((eventHandler, eventName) => {
            player.on(eventName, eventHandler);
        });
    }
    broadcastStatusChange(type, payload) {
        const runOutsideAngular = this.ngZone
            ? (fun) => this.ngZone.runOutsideAngular(fun)
            : (fun) => fun();
        runOutsideAngular(() => this.stateChange$.next({ type, payload }));
    }
}
EventsRegistry.ɵprov = ɵɵdefineInjectable({ factory: function EventsRegistry_Factory() { return new EventsRegistry(ɵɵinject(NgZone, 8)); }, token: EventsRegistry, providedIn: "root" });
EventsRegistry.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
EventsRegistry.ctorParameters = () => [
    { type: NgZone, decorators: [{ type: Optional }] }
];

class YtPlayerWrapperService {
    createInstance(htmlId, playerOptions) {
        return new YouTubePlayer(htmlId, playerOptions);
    }
}
YtPlayerWrapperService.ɵprov = ɵɵdefineInjectable({ factory: function YtPlayerWrapperService_Factory() { return new YtPlayerWrapperService(); }, token: YtPlayerWrapperService, providedIn: "root" });
YtPlayerWrapperService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];

class YtPlayerService {
    constructor(ytPlayerWrapper, eventsRegistry) {
        this.ytPlayerWrapper = ytPlayerWrapper;
        this.eventsRegistry = eventsRegistry;
    }
    get stateChange$() {
        return this.eventsRegistry.stateChange$.asObservable();
    }
    get destroyed() {
        return this.player.destroyed;
    }
    get videoId() {
        return this.player.videoId;
    }
    get player() {
        return this.ytPlayer;
    }
    set player(value) {
        this.ytPlayer = value;
    }
    init(htmlId, playerOptions) {
        this.setUpPlayer(htmlId, playerOptions);
        this.eventsRegistry.register(this.player);
    }
    load(videoId) {
        const autoplay = this.playerOptions && this.playerOptions.autoplay;
        this.player.load(videoId, autoplay);
    }
    play() {
        this.player.play();
    }
    pause() {
        this.player.pause();
    }
    stop() {
        this.player.stop();
    }
    seek(seconds) {
        this.player.seek(seconds);
    }
    setVolume(value) {
        if (value > 100) {
            console.warn(`Provided value ${value} exceeds max value. 100 used instead`);
        }
        this.player.setVolume(value);
    }
    getVolume() {
        return this.player.getVolume();
    }
    mute() {
        this.player.mute();
    }
    unMute() {
        this.player.unMute();
    }
    isMuted() {
        return this.player.isMuted();
    }
    setSize(width, height) {
        this.player.setSize(width, height);
    }
    setPlaybackRate(rate) {
        this.player.setPlaybackRate(rate);
    }
    setPlaybackQuality(sugestedQuality) {
        this.player.setPlaybackQuality(sugestedQuality);
    }
    getPlaybackRate() {
        return this.player.getPlaybackRate();
    }
    getAvailablePlaybackRates() {
        return this.player.getAvailablePlaybackRates();
    }
    getDuration() {
        return this.player.getDuration();
    }
    getBufferingProgress() {
        return this.player.getProgress();
    }
    getState() {
        return this.player.getState();
    }
    getCurrentTime() {
        return this.player.getCurrentTime();
    }
    destroy() {
        this.player.destroy();
    }
    setUpPlayer(htmlId, playerOptions) {
        this.playerOptions = playerOptions;
        this.player = this.ytPlayerWrapper.createInstance(htmlId, playerOptions);
    }
}
YtPlayerService.ɵprov = ɵɵdefineInjectable({ factory: function YtPlayerService_Factory() { return new YtPlayerService(ɵɵinject(YtPlayerWrapperService), ɵɵinject(EventsRegistry)); }, token: YtPlayerService, providedIn: "root" });
YtPlayerService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
YtPlayerService.ctorParameters = () => [
    { type: YtPlayerWrapperService },
    { type: EventsRegistry }
];

class IdGeneratorService {
    constructor() {
        this.charCodes = { a: 65, z: 90 };
        this.idLength = 8;
    }
    generate() {
        return [...Array(this.idLength).keys()]
            .reduce(acc => `${acc}${this.getRandomLetter()}`, '');
    }
    getRandomLetter() {
        const charCode = this.getRandomIntInRange(this.charCodes.a, this.charCodes.z);
        return String.fromCharCode(charCode);
    }
    getRandomIntInRange(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
}
IdGeneratorService.ɵprov = ɵɵdefineInjectable({ factory: function IdGeneratorService_Factory() { return new IdGeneratorService(); }, token: IdGeneratorService, providedIn: "root" });
IdGeneratorService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];

class QueueService {
    constructor() {
        this.queue = new Map();
    }
    enqueue(task) {
        const key = this.generateKey();
        this.addTask(key, task);
    }
    dequeue() {
        const task = this.getAndRemoveFromQueue();
        this.execute(task);
    }
    addTask(key, task) {
        this.queue.set(key, task);
    }
    generateKey() {
        return this.queue.size + 1;
    }
    getAndRemoveFromQueue() {
        const latestKey = this.queue.size;
        const task = this.queue.get(latestKey);
        this.queue.delete(latestKey);
        return task;
    }
    execute(task) {
        if (task && typeof task === 'function') {
            task();
        }
    }
}
QueueService.ɵprov = ɵɵdefineInjectable({ factory: function QueueService_Factory() { return new QueueService(); }, token: QueueService, providedIn: "root" });
QueueService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];

class YtPlayerComponent {
    constructor(ytPlayerService, idGeneratorService, queueService) {
        this.ytPlayerService = ytPlayerService;
        this.idGeneratorService = idGeneratorService;
        this.queueService = queueService;
        this.stateChange = new EventEmitter$1();
        this.errorMessage = 'Player not initialized. Did you specify [videoId] input property of yt-player-component?';
    }
    ngOnInit() {
        this.setUpElementRefId();
        this.dequeuePlayerInitialization();
        this.loadVideo();
        this.subscribeToStateChanges();
    }
    ngOnChanges() {
        if (!this.videoIdIsDefined()) {
            return;
        }
        else if (this.elementRefIdIsDefined()) {
            this.reloadVideo();
        }
        else {
            this.enqueuePlayerInitialization();
        }
    }
    ngOnDestroy() {
        this.ytPlayerService.destroy();
        this.stateChangeSubscription.unsubscribe();
    }
    setUpElementRefId() {
        const id = this.idGeneratorService.generate();
        this.ytHtmlElementHook.nativeElement.setAttribute('id', id);
    }
    videoIdIsDefined() {
        return !!this.videoId;
    }
    elementRefIdIsDefined() {
        return (this.ytHtmlElementHook.nativeElement &&
            this.ytHtmlElementHook.nativeElement.getAttribute('id'));
    }
    initializePlayer() {
        const id = this.ytHtmlElementHook.nativeElement.getAttribute('id');
        this.ytPlayerService.init(`#${id}`, this.options);
    }
    loadVideo() {
        try {
            this.ytPlayerService.load(this.videoId);
        }
        catch (err) {
            console.error(this.errorMessage);
        }
    }
    reloadVideo() {
        this.ytPlayerService.destroy();
        this.initializePlayer();
        this.loadVideo();
    }
    enqueuePlayerInitialization() {
        this.queueService.enqueue(() => this.initializePlayer());
    }
    dequeuePlayerInitialization() {
        this.queueService.dequeue();
    }
    subscribeToStateChanges() {
        this.stateChangeSubscription = this.ytPlayerService.stateChange$.subscribe((stateChange) => this.stateChange.emit(stateChange));
    }
}
YtPlayerComponent.decorators = [
    { type: Component, args: [{
                selector: 'yt-player',
                template: ` <div #ytHtmlElementHook></div> `
            },] }
];
YtPlayerComponent.ctorParameters = () => [
    { type: YtPlayerService },
    { type: IdGeneratorService },
    { type: QueueService }
];
YtPlayerComponent.propDecorators = {
    videoId: [{ type: Input }],
    options: [{ type: Input }],
    stateChange: [{ type: Output }],
    ytHtmlElementHook: [{ type: ViewChild, args: ['ytHtmlElementHook', { static: true },] }]
};

class YtPlayerAngularModule {
}
YtPlayerAngularModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [YtPlayerComponent],
                exports: [YtPlayerComponent],
            },] }
];

/*
 * Public API Surface of yt-player-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PlaybackQuality, PlayerState, StateChangeType, YtPlayerAngularModule, YtPlayerComponent, YtPlayerService, YtPlayerWrapperService as ɵa, EventsRegistry as ɵb, IdGeneratorService as ɵc, QueueService as ɵd };
//# sourceMappingURL=yt-player-angular.js.map
